@@ -1,6 +1,6 @@
 use std::{net::{SocketAddr}, sync::{Arc, atomic::{AtomicU8, AtomicI64}}};
 use rand::Rng;
-use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel}, time::{sleep, timeout}};
+use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel, Notify}, time::{sleep, timeout}};
 
 use tokio::sync::mpsc::{Sender, Receiver};
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -8,6 +8,7 @@ use crate::error::{Result, RaknetError};
 
 use crate::{packet::*, utils::*, arq::*, raknet_log};
 
+/// Raknet socket wrapper with local and remote.
 pub struct RaknetSocket{
     local_addr : SocketAddr,
     peer_addr : SocketAddr,
@@ -18,11 +19,14 @@ pub struct RaknetSocket{
     connected : Arc<AtomicBool>,
     last_heartbeat_time : Arc<AtomicI64>,
     enable_loss : Arc<AtomicBool>,
-    loss_rate : Arc<AtomicU8>
+    loss_rate : Arc<AtomicU8>,
+    incomming_notify : Arc<Notify>,
 }
 
 impl RaknetSocket {
-    
+    /// Create a Raknet Socket from a UDP socket with an established Raknet connection
+    /// 
+    /// This method is used for RaknetListener, users of the library should not care about it.
     pub fn from(addr : &SocketAddr , s : &Arc<UdpSocket> ,receiver : Receiver<Vec<u8>> , mtu : u16 , collecter : Arc<Mutex<Sender<SocketAddr>>>) -> Self {
 
         let (user_data_sender , user_data_receiver) =  channel::<Vec<u8>>(100);
@@ -37,14 +41,15 @@ impl RaknetSocket {
             connected : Arc::new(AtomicBool::new(true)),
             last_heartbeat_time : Arc::new(AtomicI64::new(cur_timestamp_millis())),
             enable_loss : Arc::new(AtomicBool::new(false)),
-            loss_rate : Arc::new(AtomicU8::new(0))
+            loss_rate : Arc::new(AtomicU8::new(0)),
+            incomming_notify : Arc::new(Notify::new()),
         };
         ret.start_receiver(receiver , user_data_sender);
         ret.start_tick(Some(collecter));
         ret
     }
 
-    async fn handle (frame : &FrameSetPacket , peer_addr : &SocketAddr , local_addr : &SocketAddr , sendq : &Mutex<SendQ> , user_data_sender : &Sender<Vec<u8>>) -> Result<bool> {
+    async fn handle (frame : &FrameSetPacket , peer_addr : &SocketAddr , local_addr : &SocketAddr , sendq : &Mutex<SendQ> , user_data_sender : &Sender<Vec<u8>> , incomming_notify : &Notify) -> Result<bool> {
         match PacketID::from(frame.data[0])? {
             PacketID::ConnectionRequest => {
                 let packet = read_packet_connection_request(frame.data.as_slice()).await?;
@@ -80,6 +85,8 @@ impl RaknetSocket {
                 //i dont know why incomming packet after always follow a connected ping packet in minecraft bedrock 1.18.12.
                 let buf = write_packet_connected_ping(&ping).await?;
                 sendq.insert(Reliability::Unreliable ,&buf)?;
+                raknet_log!("incomming notified");
+                incomming_notify.notify_one();
             }
             PacketID::NewIncomingConnection => {
                 let _packet = read_packet_new_incomming_connection(frame.data.as_slice()).await?;
@@ -110,12 +117,6 @@ impl RaknetSocket {
         }
         Ok(true)
     }
-
-    pub fn set_loss_rate(&mut self ,stage : u8){
-        self.enable_loss.store(true, Ordering::Relaxed);
-        self.loss_rate.store(stage, Ordering::Relaxed);
-
-    }
     
     async fn sendto(s : &UdpSocket , buf : &[u8] , target : &SocketAddr , enable_loss : &AtomicBool , loss_rate : &AtomicU8) -> tokio::io::Result<usize>{
         if enable_loss.load(Ordering::Relaxed){
@@ -128,7 +129,19 @@ impl RaknetSocket {
         }
         s.send_to(buf, target).await
     }
-
+    
+    /// Connect to a Raknet server and return a Raknet socket
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// socket.send(&[0xfe], Reliability::ReliableOrdered).await.unwrap();
+    /// let buf = socket.recv().await.unwrap();
+    /// if buf[0] == 0xfe{
+    ///    //do something
+    /// }
+    /// socket.close().await.unwarp(); // you need to manually close raknet connection
+    /// ```
     pub async fn connect(addr : &SocketAddr) -> Result<Self>{
 
         let guid : u64 = rand::random();
@@ -254,11 +267,16 @@ impl RaknetSocket {
             connected,
             last_heartbeat_time : Arc::new(AtomicI64::new(cur_timestamp_millis())),
             enable_loss : Arc::new(AtomicBool::new(false)),
-            loss_rate : Arc::new(AtomicU8::new(0))
+            loss_rate : Arc::new(AtomicU8::new(0)),
+            incomming_notify : Arc::new(Notify::new()),
         };
 
         ret.start_receiver(receiver , user_data_sender);
         ret.start_tick(None);
+
+        raknet_log!("wait incomming notify");
+        ret.incomming_notify.notified().await;
+        
         Ok(ret)
     }
 
@@ -272,6 +290,7 @@ impl RaknetSocket {
         let enable_loss = self.enable_loss.clone();
         let loss_rate = self.loss_rate.clone();
         let last_heartbeat_time = self.last_heartbeat_time.clone();
+        let incomming_notify = self.incomming_notify.clone();
         tokio::spawn(async move {
             loop{
                 if !connected.load(Ordering::Relaxed){
@@ -335,7 +354,7 @@ impl RaknetSocket {
                         recvq.insert(frame).unwrap();
 
                         for f in recvq.flush(&peer_addr){
-                            if !RaknetSocket::handle(&f , &peer_addr ,&local_addr, &sendq, &user_data_sender).await.unwrap(){
+                            if !RaknetSocket::handle(&f , &peer_addr ,&local_addr, &sendq, &user_data_sender , &incomming_notify).await.unwrap(){
                                 connected.store(false, Ordering::Relaxed);
                                 is_break = true;
                                 break;
@@ -448,6 +467,32 @@ impl RaknetSocket {
         });
     }
 
+    /// Close Raknet Socket
+    /// 
+    /// The Raknet Socket needs to be closed manually, and if no valid data packets are received for more than 1 minute, the Raknet connection will be closed automatically. 
+    /// This method can be called repeatedly.
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// socket.close().await.unwarp();
+    /// ```
+    pub async fn close(&mut self) -> Result<()>{
+
+        if self.connected.load(Ordering::Relaxed){
+            self.sendq.lock().await.insert(Reliability::Reliable, &[PacketID::Disconnect.to_u8()])?;
+            self.connected.store(false, Ordering::Relaxed);
+        }
+        Ok(())
+    }
+
+    /// Unconnected ping a Raknet Server and return latency
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let latency = socket::RaknetSocket::ping("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// assert!((0..10).contains(&latency));
+    /// ```
     pub async fn ping(addr : &SocketAddr) -> Result<i64> {
         let packet = PacketUnconnectedPing{
             time: cur_timestamp_millis(),
@@ -478,12 +523,17 @@ impl RaknetSocket {
         Ok(pong.time - packet.time)
     }
 
-    pub async fn close(&mut self) -> Result<()>{
-        self.sendq.lock().await.insert(Reliability::Reliable, &[PacketID::Disconnect.to_u8()])?;
-        self.connected.store(false, Ordering::Relaxed);
-        Ok(())
-    }
-
+    /// Send a packet
+    /// 
+    /// packet must be `0xfe` as the first byte, using other values of bytes may cause unexpected errors.
+    /// 
+    /// Except Reliability::ReliableOrdered, all other reliability packets must be less than MTU - 60 (default 1340 bytes), otherwise RaknetError::PacketSizeExceedMTU will be returned
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// socket.send(&[0xfe], Reliability::ReliableOrdered).await.unwrap();
+    /// ```
     pub async fn send(&mut self , buf : &[u8] , r : Reliability) ->Result<()> {
 
         if !self.connected.load(Ordering::Relaxed){
@@ -501,6 +551,16 @@ impl RaknetSocket {
         Ok(())
     }
 
+    /// Recv a packet
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// let buf = socket.recv().await.unwrap();
+    /// if buf[0] == 0xfe{
+    ///    //do something
+    /// }
+    /// ```
     pub async fn recv(&mut self) -> Result<Vec<u8>> {
 
         if !self.connected.load(Ordering::Relaxed){
@@ -519,11 +579,40 @@ impl RaknetSocket {
 
     }
 
+    /// Returns the socket address of the remote peer of this Raknet connection.
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// assert_eq!(socket.peer_addr().unwrap(), SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 19132)));
+    /// ```
     pub fn peer_addr(&self) -> Result<SocketAddr>{
         Ok(self.peer_addr)
     }
 
+    /// Returns the socket address of the local half of this Raknet connection.
+    /// 
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// assert_eq!(socket.local_addr().unwrap().ip(), IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)));
+    /// ```
     pub fn local_addr(&self) -> Result<SocketAddr>{
         Ok(self.local_addr)
     }
+
+    /// Set the packet loss rate and use it for testing
+    /// 
+    /// The `stage` parameter ranges from 1 to 10, indicating a packet loss rate of 10% to 20%.
+    /// # Example
+    /// ```ignore
+    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// // set 80% loss packet rate.
+    /// socket.set_loss_rate(8);
+    /// ```
+    pub fn set_loss_rate(&mut self ,stage : u8){
+        self.enable_loss.store(true, Ordering::Relaxed);
+        self.loss_rate.store(stage, Ordering::Relaxed);
+
+    }
 }
\ No newline at end of file
